from unittest import TestCase

from sanka import sanka
from sanka import YaDead


class SankaTest(TestCase):

    def test___str__(self):

        @sanka
        def function():
            pass

        assert str(function) == 'function'

    def test_count_calls_for_no_arg_function(self):

        @sanka
        def function():
            pass

        num_calls = 2

        for i in range(num_calls):
            function()

        assert function(YaDead) == num_calls

    def test_count_calls_for_function_with_args(self):

        @sanka
        def function(takes, some_args):
            pass

        num_calls = 2

        for i in range(num_calls):
            function('takes', 'some_args')

        assert function(YaDead) == num_calls

    def test_count_calls_for_multiple_functions(self):

        @sanka
        def function1():
            pass
        @sanka
        def function2():
            pass

        num_calls_for_first_function = 2
        num_calls_for_second_function = 3

        for i in range(num_calls_for_first_function):
            function1()
        for i in range(num_calls_for_second_function):
            function2()

        assert function1(YaDead) == num_calls_for_first_function
        assert function2(YaDead) == num_calls_for_second_function

    def test_sanka_decorator_can_detect_no_function_calls(self):
        @sanka
        def function():
            pass

        assert function(YaDead) == 0

    def test_sanka_decorator_only_accepts_function_as_callback(self):

        with self.assertRaises(TypeError):
            @sanka(callback='not a function')
            def function():
                pass

    def test_sanka_decorator_can_accept_callback(self):
        @sanka(callback=lambda count: count)
        def function():
            pass

        num_calls = 2

        for i in range(num_calls):
            function()

        assert function(YaDead) == num_calls

    def test_sanka_can_call_back_every_call(self):
        side_effect_counts = []

        @sanka(
            callback=lambda count: side_effect_counts.append(count),
            only_callback_when_dead = False
        )
        def function():
            pass

        num_calls = 2

        for i in range(num_calls):
            function()

        assert side_effect_counts == list(range(1, num_calls + 1))
        assert function(YaDead) == num_calls




from typing import Callable
from typing import TypeVar


T = TypeVar('T')

class Sanka:

    def __init__(
        self,
        func: Callable,
        callback: Callable[[int], T] = None,
        only_callback_when_dead: bool = True
    ):
        if callback and not callable(callback):
            raise TypeError(f'Callback must be Callable not "{type(callback).__name__}"')

        self.func = func
        self.call_count = 0
        self.callback = callback
        self.only_callback_when_dead = only_callback_when_dead

    def __call__(self, *args, **kwargs):
        if YaDead in args:
            if self.callback and self.only_callback_when_dead:
                self.callback(self.call_count)
            return self.call_count
        else:
            result = self.func(*args, **kwargs)
            self.call_count += 1

            if self.callback and not self.only_callback_when_dead:
                self.callback(self.call_count)

            return result

    def __str__(self):
        return self.func.__name__


def sanka(
    function: Callable = None,
    callback: Callable[[int], T] = None,
    only_callback_when_dead: bool = True
):
    if function:
        return Sanka(function)
    else:
        def wrapper(function):
            return Sanka(function, callback, only_callback_when_dead)
        return wrapper


class YaDead:
    pass




from os import environ as envvars
from pathlib import Path
from setuptools import find_packages
from setuptools import setup


setup(
    name='sanka',
    description='a function decorator for surfacing dead code',
    license='MIT',
    long_description=Path(__file__).absolute().parent.joinpath('README.md').read_text(),
    long_description_content_type='text/markdown',
    use_scm_version={'local_scheme': 'no-local-version'} if envvars.get('LOCAL_VERSION_SCHEME') else True,
    author='Emmanuel I. Obi',
    maintainer='Emmanuel I. Obi',
    maintainer_email='withtwoemms@gmail.com',
    url='https://github.com/withtwoemms/sanka',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    package=find_packages(),
    setup_requires=[
        'setuptools_scm==5.0.1'
    ],
)



import nox


PROJECT_NAME = 'sanka'
VENV = f'{PROJECT_NAME}-venv'
TESTDIR = '.'

supported_python_versions = [
    '3.6',
    '3.7',
    '3.8',
    '3.9',
    '3.10',
]

@nox.session(name=VENV, python=supported_python_versions, reuse_venv=True)
def tests(session):
    session.install('.')
    session.install('-r', 'requirements.txt')
    session.run(
        'python', '-m',
        'coverage', 'run', '--branch',
        '--omit', '.nox/*,noxfile.py,setup.py,test*',
        '--source', '.',
        '-m', 'unittest', 'discover',
        '-s', TESTDIR
    )
    session.run('coverage', 'report', '-m')
    session.run('coverage', 'xml')


@nox.session(name=f'build-{PROJECT_NAME}', reuse_venv=True)
def build(session):
    session.install('.')
    session.run('python', 'setup.py', 'sdist')


# TODO (withtwoemms) -- leverage gitpython to show version on-demand

